/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition;

import com.rabbitmq.client.AMQP;
import com.vii.brillien.ignition.classloading.PresenceClassLoader;
import com.vii.brillien.ignition.transport.BrillienMediator;
import com.vii.brillien.ignition.transport.amqp.AmqpMediator;
import com.vii.brillien.ignition.transport.jsonrpc.JsonRpcMediator;
import com.vii.brillien.ignition.transport.mq.MqMediator;
import com.vii.brillien.ignition.transport.xmpp.XmppMediator;
import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.transport.Manager;
import com.vii.streamline.services.ThreadServices;
import com.vii.streamline.services.db.DBServices;
import com.vii.streamline.services.io.configuration.ConfigurationReader;
import com.vii.streamline.services.io.configuration.ConfigurationTreeNode;
import com.vii.streamline.services.reflection.ReflectionServices;
import com.vii.streamline.structures.collections.InnerMap;
import org.slf4j.impl.BrillienLoggerAdapter;

import java.io.IOException;
import java.sql.Connection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.*;

/**
 * Emvironmental properties of a running Brillien like timout settings, server name, ports values, etc.
 */
public class BrillienContext {

    /**
     * Constant for MQ-based transport layer
     */
    public static final int                 TRANSPORT_MQ        = 1;
    /**
     * Constant for AMQP-based transport layer
     */
    public static final int                 TRANSPORT_AMQP      = 2;
    /**
     * Constant for XMPP-based transport layer
     */
    public static final int                 TRANSPORT_XMPP      = 3;
    /**
     * Constant for JSON-RPC-based transport layer
     */
    public static final int                 TRANSPORT_JSON_RPC  = 4;


    /**
     * Root of the classloader hierarchy of Brillien
     */
    public static PresenceClassLoader       primordialClassLoader;

    /**
     * Heartbeat frequency of the Brillien node instance 
     */
    public static long                      HEARTBEAT_PATIENCE;

    /**
     * Timout for flow termination. Exceeding it, the flow is considered as a broken operation and replication is started by the cluster.
     */
    public static long                      FLOW_PATIENCE;


    /**
     * Name of the Brillien instance (must be unique in a cluster)
     */
    public static String                    SERVER_NAME;

    /**
     * Version of the server
     */
    public static Float                     SERVER_VERSION;

    public static long                      COMMUNICATION_REPLY_TIMEOUT;

    public static long                      RECONNECTION_TIMEOUT;

    public static long                      RECONNECTION_ATTEMPT;

    /**
     * Configuration of the transport layer
     */
    public static String                    transportType;
    public static String                    transportProvider;
    public static Class                     transportProviderClass;
    public static HashMap<String, String>   transportProviderParameters;
    public static BrillienMediator          mediator;

    /**
     * Timout of a clean shutdown. After the "time is up" message, the unclosed service will be shut down immediately
     */
    public static long                      SHUTDOWN_TOLERANCE;

    /**
     * Size of the thread pool used by the Brillien instance to parallelize the communication.
     */
    public static int                       THREAD_POOL_SIZE;

    /**
     * Parameter for the object pool of a presence.
     * Sets the maximal count of unused presences. Overstepping this limit, the unnecessary instances will be finalized.
     */
    public static int                       MAX_IDLE_PRESENCE_COUNT;
    
    /**
     * Parameter for the object pool of a presence.
     * Sets the initial count of a presence instantiated after publishing.
     */
    public static int                       INITIAL_PRESENCE_COUNT;

    /**
     * Logger Entity for logging activiy of main processes of Brillien instance
     */
    public static Logger                    mainLogger;

    /**
     * Logger handler performing formatting.
     */
    public static Handler                   mainHandler;

    /**
     * Special Handler for exceptional cases. Write directly to the console.(Exception are logged to the mainHandler too.)
     */
    public static Handler                   throwableHandler;

    /**
     * Interest level of the main logger. Every logging request with lower level will be ingnored.
     */
    public static Level                     INTEREST_LEVEL;

    /**
     * Default level of the logging requests.
     */
    public static Level                     DEFAULT_LEVEL;

    /**
     * ID of the flow of Brillien itself 
     */
    public static String                    SUPREME_FLOW_ID;

    /**
     * ID of the root flow of components published in Brillien
     */
    public static String                    SERVICES_FLOW_ID;

    public static boolean                   SSO_ACTIVATED;
    public static String                    SSO_PROVIDER;
    public static HashMap<String, String>   SSO_PARAMETERS;

    /**
     * Default autocommit flag for those Transactors who don't specify it.
     */
    public static boolean                   DEFAULT_AUTOCOMMIT;
    
    /**
     * Default isolation level for those Transactors who don't specify it.
     */
    public static int                       DEFAULT_ISOLATION;

    /**
     * Stores all DBVendor info read from config file
     */
    public static HashMap<String, InnerMap>  DB_VENDORS;

    /**
     * Flag for the JMX service of Brillien.
     */
    public static boolean                   JMX_SERVICE_ON;

    /**
     * Port number of the JMX service of Brillien
     */
    public static int                       JMX_SERVICE_PORT;

    /**
     * Tells whether Brillien should clean the Couchdb during Startup process
     */
    public static boolean                   CLEAN_AT_STARTUP;

    /**
     * Flag marking if a couchdb is available for Flow fault-tolerance
     */
    public static boolean                   COUCHDB_BOARD;

    /**
     * URI of the Couchdb instance
     */
    public static String                    COUCHDB_URI;

    /**
     * Timecycle of the heartbeat of the Commander Implementation of Brillien. Measured in milliseconds.
     */
    public static long                      HEARTBEAT_CYCLE_MEASURE;

    /**
     * Timecycle of the cluster-checker Presence watching nodes and flows to be replicated in case of operation failure. Mesured in milliseconds.
     */
    public static long                      KEEPER_CYCLE_MEASURE;

    /**
     * Timecycle of the inner recycle Presence. Mesured in milliseconds.
     */
    public static long                      RETRIEVER_CYCLE_MEASURE;

    /**
     * Timecycle of the component Activator which check if components should be published or republished from the activator directory. Mesured in milliseconds.
     */
    public static long                      ACTIVATOR_CYCLE_MEASURE;

    /**
     * Tells whether the server should send a message to the given recipient about a successful component publishing...
     */
    public static String                    COMPONENT_PUBLISHING_RECIPIENT;



    public static boolean                   WEBSERVER_ON;
    public static String                    WEBSERVER_HOSTNAME;
    public static int                       WEBSERVER_PORT;
    public static String                    WEBSERVER_DOC_ROOT;
    public static String                    WEBSERVER_JSONRPC_SERVLET_CONTEXT;
    public static String                    WEBSERVER_JSONRPC_SERVLET_PATH;


    public static String[][]                configuration;

    static{
        HEARTBEAT_PATIENCE          = 10 * 1000;
        FLOW_PATIENCE               = 1 * 100;

        RECONNECTION_TIMEOUT        = 5000;
        RECONNECTION_ATTEMPT        = 3;
        COMMUNICATION_REPLY_TIMEOUT = 5000;

        SSO_ACTIVATED               = false;

        primordialClassLoader       = new PresenceClassLoader(
                                            ClassLoader.getSystemClassLoader()
                                      );

        JMX_SERVICE_ON              = false;

        WEBSERVER_ON                = false;
    }

    /**
     * Initializer method to read configuration file of the Brillien application server
     * @param reader XML tree structure of the config file
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     * @throws java.io.IOException
     */
    public static void initServices( ConfigurationReader reader ) throws BrillienException {
        initServices( null, reader );
    }

/**
     * Initializer method to read configuration file of the Brillien application server
     * @param loader PresenceClassLoader instance to manage all classloader-related task
     * @param reader XML tree structure of the config file
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     * @throws java.io.IOException
     */
    public static void initServices( PresenceClassLoader loader, ConfigurationReader reader ) throws BrillienException {
        primordialClassLoader   = loader;

        SERVER_NAME             = reader.getStringValue( "serverName" );
        SERVER_VERSION          = reader.getFloatValue( "serverVersion" );

        HEARTBEAT_PATIENCE      = reader.getLongValue( "heartbeatPatience" );
        FLOW_PATIENCE           = reader.getLongValue( "flowPatience" );

        transportType           = reader.getStringValue( "transportLayer", "id" );

        ConfigurationTreeNode _node = reader.getTreeNodesByNodeName( "transportLayer" )[0];

        RECONNECTION_TIMEOUT    = reader.getIntegerValue( "transportLayer", "reconnectionTimeout" );
        RECONNECTION_ATTEMPT    = reader.getIntegerValue( "transportLayer", "reconnectionAttempt" );
        COMMUNICATION_REPLY_TIMEOUT = reader.getIntegerValue( "transportLayer", "replyTimeout" );
        ConfigurationTreeNode[] nodes = reader.getTreeNodesByNodeName( "provider", "transportLayer", "providers" );
        for(ConfigurationTreeNode node : nodes){
            if( reader.getStringValue( node, "name" ).equals( transportType ) ){
                transportProvider        = reader.getStringValue( node, "class" );
                try {
                    transportProviderClass   = primordialClassLoader.loadClass(transportProvider);
                } catch (ClassNotFoundException e) {
                    throw new BrillienException("Cannot load provider class:" + transportProvider);
                }
                if( !BrillienMediator.class.isAssignableFrom( transportProviderClass ) ){
                    throw new BrillienException("All providers has to be a subclass of " + BrillienMediator.class.getSimpleName() );
                }
            }
        }
        if( transportProvider == null )
            throw new BrillienException("No proper transport provider found for:" + transportType);

        transportProviderParameters = new HashMap<String, String>();
        ConfigurationTreeNode parameters = reader.getTreeNodesByNodeName( transportType, "transportLayer", "config" )[0];
        if( transportProviderClass.equals( AmqpMediator.class ) || transportProviderClass.equals( XmppMediator.class ) ){
            transportProviderParameters.put("username", "liaison");
            transportProviderParameters.put("password", "liaison");
        }
        else if( transportProviderClass.equals( MqMediator.class ) ){
            transportProviderParameters.put("queuename", "liaison");
        }
        else if( transportProviderClass.equals( JsonRpcMediator.class ) ){
            transportProviderParameters.put("queuename", "liaison");
            transportProviderParameters.put("url", "http://localhost:8080/jsonrpc/ExchangeServlet");
        }

        for( ConfigurationTreeNode node : parameters.getChildren() ){
            transportProviderParameters.put( node.getNodeValue(), node.getContainedValue().getNodeValue() );
        }
        mediator = newMediator(); mediator.connect();

        SHUTDOWN_TOLERANCE      = reader.getLongValue( "shutdownTolerance" );

        THREAD_POOL_SIZE        = reader.getIntegerValue( "threadPoolSize" );

        JMX_SERVICE_ON          = reader.getStringValue( "jmx", "turned" ).equalsIgnoreCase("on");
        JMX_SERVICE_PORT        = reader.getIntegerValue( "jmx", "jmxPort" );

        INITIAL_PRESENCE_COUNT  = reader.getIntegerValue( "initialPresenceCount" );
        MAX_IDLE_PRESENCE_COUNT = reader.getIntegerValue( "maxIdlePresenceCount" );

        SUPREME_FLOW_ID         = reader.getStringValue( "supremeFlowID" );
        SERVICES_FLOW_ID        = reader.getStringValue( "servicesFlowID" );

        DEFAULT_AUTOCOMMIT      = reader.getBooleanValue( "defaultAutocommit" );
        try {
            DEFAULT_ISOLATION = Connection.class.getField( reader.getStringValue("defaultIsolation") ).getInt( Connection.class );
        } catch (Exception e) {
            throw new BrillienException( "Cannot retrieve isolation from config file.", e );
        }

        COUCHDB_BOARD = reader.hasValue("couchdb") && reader.getStringValue( "couchdb", "turned" ).equalsIgnoreCase("on");
        CLEAN_AT_STARTUP  = reader.hasValue("couchdb") ? reader.getBooleanValue( "couchdb", "cleanAtStartup" ) : false; 
        COUCHDB_URI = COUCHDB_BOARD
                    ? reader.getStringValue( "couchdb", "uri" )
                    : "localhost";

        try {
            BrillienLoggerAdapter.mainLogger = mainLogger =
                    BrillienLoggerAdapter.mainLogger == null
                        ? Logger.getLogger( SERVER_NAME )
                        : BrillienLoggerAdapter.mainLogger;

            mainLogger.setUseParentHandlers( false );
            mainLogger.setLevel( Level.ALL );
            
            mainHandler = getHandler( reader );
            if( mainHandler != null ){
                mainLogger.addHandler( mainHandler );

                Formatter formatter = getFormatter( reader );
                if( formatter != null )
                    mainHandler.setFormatter( formatter );

                throwableHandler = getConsoleHandler( );
                throwableHandler.setFilter( new Filter(){
                    public boolean isLoggable(LogRecord logRecord) {
                        return logRecord != null && logRecord.getMessage() != null && logRecord.getMessage().equals("THROW");
                    }
                } );
                //if( !(mainHandler instanceof ConsoleHandler) ){
                basicLog( Level.CONFIG, "Throwable instances will be directed to console too...");

                mainLogger.addHandler( throwableHandler );
                //}
            }
        } catch (Exception e) {
            throw new BrillienException( "Cannot set-up the logging subsystem.", e );
        }
        try {
            mainHandler.setLevel( INTEREST_LEVEL =
                (Level) ReflectionServices.getFieldValue( Level.class, Level.class,
                    reader.getStringValue( "debug", "interestLevel" )
                )
            );
        } catch (Exception e) {
            throw new BrillienException( "Cannot set the default interest level of the Logger.", e );
        }

        try {
            DEFAULT_LEVEL = (Level)ReflectionServices.getFieldValue( Level.class, Level.class,
                    reader.getStringValue( "debug", "defaultLevel" )
            );
        } catch (Exception e) {
            throw new BrillienException( "Cannot set the default logging level of the Logger.", e );
        }

        try {
            HEARTBEAT_CYCLE_MEASURE = reader.getLongValue( "heartbeatCycle" );

            KEEPER_CYCLE_MEASURE = reader.getLongValue( "keeperCycle" );

            RETRIEVER_CYCLE_MEASURE = reader.getLongValue( "retrieverCycle" );

            ACTIVATOR_CYCLE_MEASURE = reader.getLongValue( "activatorCycle" );
        } catch (Exception e) {
            throw new BrillienException( "Cannot set the default timcycles of heartbeat, keeper, retriever, activator.", e );
        }

        DEFAULT_AUTOCOMMIT  = reader.getBooleanValue( "dataSource", "defaultAutocommit" );
        try {
            DEFAULT_ISOLATION   = Connection.class.getField( reader.getStringValue("dataSource", "defaultIsolation") ).getInt( Connection.class );
        } catch (Exception e) {
            throw new BrillienException( "Cannot set the default value of DEFAULT_ISOLATION", e );
        }
        DB_VENDORS = new HashMap<String, InnerMap>();
        ConfigurationTreeNode[] vnodes = reader.getTreeNodesByNodeName( "vendor", "dataSource", "vendors" );
        for (ConfigurationTreeNode node : vnodes) {
            try{
                InnerMap<String, Object> map = new InnerMap<String, Object>();
                map.put( "connectionURL", reader.getStringValue( node, "connectionURL" ) );
                map.put( "dsdriver", reader.getStringValue( node, "dsdriver" ) );
                map.put( "xadriver", reader.getStringValue( node, "xadriver" ) );
                map.put( "defaultPort", Integer.valueOf( reader.getStringValue( node, "defaultPort" ) ) );
                DB_VENDORS.put( reader.getStringValue( node, "name" ), map );
            } catch(Exception e){ throw new BrillienException( "Invalid DBVendor description", e ); }
        }
        DBServices.initVendors( DB_VENDORS );

        if( reader.hasValue("sso") ){
            try{
                SSO_ACTIVATED = reader.hasValue("sso", "enabled") ? reader.getBooleanValue("sso", "enabled"): false;
                SSO_PROVIDER = reader.getStringValue( "sso", "providerPresence" );
                SSO_PARAMETERS = new HashMap<String, String>();
                ConfigurationTreeNode[] sso_nodes = reader.getTreeNodesByNodeName( "parameter", "sso" );
                for (ConfigurationTreeNode sso_node : sso_nodes)
                    SSO_PARAMETERS.put( reader.getStringValue( sso_node, "name" ), reader.getStringValue( sso_node, "value" ) );

                basicLog( Level.CONFIG, "SSO activated: " + SSO_ACTIVATED );
                basicLog( Level.CONFIG, "SSO provider: " + SSO_PROVIDER );
            } catch(Exception e){ throw new BrillienException( "Invalid SSO description", e ); }
        }

        COMPONENT_PUBLISHING_RECIPIENT = reader.hasValue("componentPublishingRecipient")
                ? reader.getStringValue( "componentPublishingRecipient" )
                : null;


        if( reader.hasValue("webserver") ){
            try{
                WEBSERVER_ON = reader.hasValue("webserver", "enabled") ? reader.getBooleanValue("webserver", "enabled"): false;
                if( WEBSERVER_ON ){
                    WEBSERVER_HOSTNAME = reader.getStringValue("webserver", "hostName");
                    WEBSERVER_PORT = reader.getIntegerValue("webserver", "port");
                    WEBSERVER_DOC_ROOT = reader.getStringValue("webserver", "docRoot");
                    WEBSERVER_JSONRPC_SERVLET_CONTEXT = reader.getStringValue("webserver", "jsonRpcContext");
                    WEBSERVER_JSONRPC_SERVLET_PATH = reader.getStringValue("webserver", "jsonRpcServletPath");
                }
            } catch(Exception e){ throw new BrillienException( "Invalid Webserver description", e ); }
        }

        fillConfiguration();

        createAccountFor( "slinker" );
    }

    private static void fillConfiguration(){
        List<String[]> list = new LinkedList<String[]>();

        list.add( new String[]{ "HEARTBEAT_PATIENCE", BrillienContext.HEARTBEAT_PATIENCE+"" } );
        list.add( new String[]{ "FLOW_PATIENCE", BrillienContext.FLOW_PATIENCE+"" } );
        list.add( new String[]{ "SERVER_NAME", BrillienContext.SERVER_NAME+"" } );
        list.add( new String[]{ "SERVER_VERSION", BrillienContext.SERVER_VERSION+"" } );
        list.add( new String[]{ "Transport Type", BrillienContext.transportType+"" } );
        list.add( new String[]{ "Transport Provider", BrillienContext.transportProvider+"" } );
        list.add( new String[]{ "Transport Provider Parameters", BrillienContext.transportProviderParameters+"" } );

        list.add( new String[]{ "SHUTDOWN_TOLERANCE", BrillienContext.SHUTDOWN_TOLERANCE+"" } );
        list.add( new String[]{ "THREAD_POOL_SIZE", BrillienContext.THREAD_POOL_SIZE+"" } );
        list.add( new String[]{ "HEARTBEAT_PATIENCE", BrillienContext.HEARTBEAT_PATIENCE+"" } );
        list.add( new String[]{ "MAX_IDLE_PRESENCE_COUNT", BrillienContext.MAX_IDLE_PRESENCE_COUNT+"" } );
        list.add( new String[]{ "INITIAL_PRESENCE_COUNT", BrillienContext.INITIAL_PRESENCE_COUNT+"" } );
        list.add( new String[]{ "INTEREST_LEVEL", BrillienContext.INTEREST_LEVEL+"" } );
        list.add( new String[]{ "DEFAULT_LEVEL", BrillienContext.DEFAULT_LEVEL+"" } );
        list.add( new String[]{ "SUPREME_FLOW_ID", BrillienContext.SUPREME_FLOW_ID+"" } );
        list.add( new String[]{ "SERVICES_FLOW_ID", BrillienContext.SERVICES_FLOW_ID+"" } );
        list.add( new String[]{ "DEFAULT_AUTOCOMMIT", BrillienContext.DEFAULT_AUTOCOMMIT+"" } );
        list.add( new String[]{ "DEFAULT_ISOLATION", BrillienContext.DEFAULT_ISOLATION+"" } );
        list.add( new String[]{ "COUCHDB_BOARD", BrillienContext.COUCHDB_BOARD+"" } );
        list.add( new String[]{ "COUCHDB_URI", BrillienContext.COUCHDB_URI+"" } );
        list.add( new String[]{ "HEARTBEAT_CYCLE_MEASURE", BrillienContext.HEARTBEAT_CYCLE_MEASURE+"" } );
        list.add( new String[]{ "KEEPER_CYCLE_MEASURE", BrillienContext.KEEPER_CYCLE_MEASURE+"" } );
        list.add( new String[]{ "RETRIEVER_CYCLE_MEASURE", BrillienContext.RETRIEVER_CYCLE_MEASURE+"" } );
        list.add( new String[]{ "ACTIVATOR_CYCLE_MEASURE", BrillienContext.ACTIVATOR_CYCLE_MEASURE+"" } );
        list.add( new String[]{ "COMMUNICATION_REPLY_TIMEOUT", BrillienContext.COMMUNICATION_REPLY_TIMEOUT+"" } );
        list.add( new String[]{ "COMPONENT_PUBLISHING_RECIPIENT", BrillienContext.COMPONENT_PUBLISHING_RECIPIENT+"" } );

        configuration = list.toArray( new String[ list.size() ][] );
    }

    private static Formatter getFormatter( ConfigurationReader reader ) throws IOException {
        String formatterClass = reader.getStringValue( "debug", "handler", "formatter", "className" );
        if( formatterClass.equals("XMLFormatter") ){
            return getXMLFormatter( reader );
        } else if( formatterClass.equals("SimpleFormatter") ){
            return getSimpleFormatter( reader );
        }
        return null;
    }
    private static Formatter getXMLFormatter( ConfigurationReader reader ) throws IOException {
        return new XMLFormatter();
    }
    private static Formatter getSimpleFormatter( ConfigurationReader reader ) throws IOException {
        return new SimpleFormatter();
    }
    private static Handler getHandler( ConfigurationReader reader ) throws IOException {
        String handlerClass = reader.getStringValue( "debug", "handler", "className" );
        if( handlerClass.equals("FileHandler") ){
            return getFileHandler( reader );
        } else if( handlerClass.equals("ConsoleHandler") ){
            return getConsoleHandler( );
        } else if( handlerClass.equals("SocketHandler") ){
            return getSocketHandler( reader );
        }
        return null;
    }
    private static Handler getFileHandler( ConfigurationReader reader ) throws IOException {
        try{
            return new FileHandler(
                reader.getStringValue( "debug", "handler", "parameters", "pattern" ),
                reader.getIntegerValue( "debug", "handler", "parameters", "limit" ),
                reader.getIntegerValue( "debug", "handler", "parameters", "count" )
            );
        } finally{
            basicLog( Level.CONFIG, "Logs will be directed to file: " + reader.getStringValue( "debug", "handler", "parameters", "pattern" ) );
        }
    }
    private static Handler getSocketHandler( ConfigurationReader reader ) throws IOException {
        try{
            return new SocketHandler(
                reader.getStringValue( "debug", "handler", "parameters", "host" ),
                reader.getIntegerValue( "debug", "handler", "parameters", "port" )
            );
        } finally{
            basicLog( Level.CONFIG, "Logs will be directed to socket: " + reader.getStringValue( "debug", "handler", "parameters", "host" ) + ":" + reader.getIntegerValue( "debug", "handler", "parameters", "port" ) );
        }
    }
    private static Handler getConsoleHandler( ) throws IOException {
        try{
            ConsoleHandler c = new ConsoleHandler();
            c.setLevel( Level.ALL );
            return c;
        } finally{
            basicLog( Level.CONFIG, "Logs will be directed to console." );
        }
    }


    /**
     * The logger method used by inner services of the application server
     * @param level level of the information to be logged
     * @param msg message of the log record
     * @param parameters parameters of the log record
     */
    public static void basicLog( Level level, String msg, Object... parameters ){
        if( mainLogger != null )
            mainLogger.logp( level, "Brillien System", "SystemLog", msg, parameters );
        else{
            System.out.println( "Message:" + msg );
            for( Object s : parameters )
                System.out.println("\tParameter:" + s );
        }

    }

    /**
     * Initiates the transport functionality by the given embedded type of transport layer
     */
    public static void initTransportLayer( int transportLayerType ){
        initTransportLayer( transportLayerType, null );
    }

    /**
     * Initiates the transport functionality by the given embedded type of transport layer
     */
    public static void initTransportLayer( int transportLayerType, HashMap<String, String> props ){
        RECONNECTION_TIMEOUT = 5000;
        RECONNECTION_ATTEMPT = 3;
        COMMUNICATION_REPLY_TIMEOUT = 5000;
        transportProviderParameters = new HashMap<String, String>();
        if( transportLayerType == TRANSPORT_MQ ){
            transportType = "mq";
            transportProviderClass = MqMediator.class;
            transportProviderParameters.put("queuename", "liaison");
        }
        else if( transportLayerType == TRANSPORT_AMQP ){
            transportType = "amqp";
            transportProviderClass = AmqpMediator.class;
            transportProviderParameters.put("host", "localhost");
            transportProviderParameters.put("port", AMQP.PROTOCOL.PORT +"" );
            transportProviderParameters.put("username", "liaison");
            transportProviderParameters.put("password", "liaison");
        }
        else if( transportLayerType == TRANSPORT_XMPP ){
            transportType = "xmpp";
            transportProviderClass = XmppMediator.class;
            transportProviderParameters.put("host", "127.0.0.1");
            transportProviderParameters.put("port", "5222");
            transportProviderParameters.put("username", "liaison");
            transportProviderParameters.put("password", "liaison");
        }
        else if( transportLayerType == TRANSPORT_JSON_RPC ){
            transportType = "jsonrpc";
            transportProviderClass = JsonRpcMediator.class;
            transportProviderParameters.put("queuename", "liaison");
            transportProviderParameters.put("url", "http://localhost:8080/jsonrpc/ExchangeServlet");
        }
        if( props != null )
            for( String key : props.keySet() )
                transportProviderParameters.put( key, props.get(key) );
    }

    public static void cleanup(){
        ThreadServices.stop();
    }

    /**
     * Creates a new mediator based on the properties associated with the transport provider
     */
    public static BrillienMediator newMediator() throws BrillienException {
        return newMediator( transportProviderParameters.get("username"), transportProviderParameters.get("password") );
    }

    /**
     * Creates a new mediator based on the properties associated with the transport provider
     */
    public static BrillienMediator newMediator( String username ) throws BrillienException {
        return newMediator( username, username );
    }

    /**
     * Creates a new mediator based on the properties associated with the transport provider
     */
    public static BrillienMediator newMediator( String username, String password ) throws BrillienException {
        BrillienMediator mediator = null;
        try {
            mediator = (BrillienMediator)transportProviderClass.newInstance();

            for( String s : transportProviderParameters.keySet() ){
                ReflectionServices.setFieldValueThroughSetMethod(
                        transportProviderClass, mediator,
                        s,
                        (
                                s.equals("username") ? username :
                                        s.equals("password") ? password :
                                                s.equals("queuename") && username != null ? username :
                                                        transportProviderParameters.get(s)
                        )
                );
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BrillienException( "Cannot create instance" );
        }

        return mediator;
    }

    /**
     * The logger method used by inner services of the application server
     * @param level level of the information to be logged
     * @param msg message of the log record
     * @param parameters parameters of the log record
     */
    public static void systemLog( Level level, String msg, Object... parameters ){
        if( mainLogger != null )
            mainLogger.log( level, msg, parameters );
        else
            System.out.println( msg );
    }

    /**
     * This method used by inner services of the application server to log exceptions
     * @param sourceClass sourceClass
     * @param sourceMethod sourceMethod
     * @param thrown exception occured
     */
    public static void exceptionLog( String sourceClass, String sourceMethod, Throwable thrown ){
        if( mainLogger != null )
            mainLogger.throwing( sourceClass, sourceMethod, thrown );
        else
            thrown.printStackTrace();
    }

    public static void createAccountFor(String name) throws BrillienException {
        if( mediator instanceof Manager){
            systemLog( Level.FINE, "Creating account for : " + name );
            BrillienMediator _mediator = newMediator( name, name );
            ((Manager)mediator).createAccountFor( _mediator );
        }
    }

}
