/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.transport.Communication;
import com.vii.brillien.kernel.axiom.transport.MessageProcessor;
import com.vii.streamline.services.reflection.ReflectionServices;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Method;

import static com.vii.streamline.services.reflection.ReflectionServices.invokeMethod;

/**
 * Simple {@MessageProcessor} implementation transforming all incoming message into method calls using reflection
 */
public abstract class BrillienCallbackHandler implements MessageProcessor<BrillienCommunication> {

    protected BrillienCommunication comm;
    protected Logger                logger;

    public BrillienCallbackHandler() {
        this.logger = LoggerFactory.getLogger(this.getClass());
    }

    public BrillienCallbackHandler(Logger logger) {
        this.logger = logger;
    }

    @Override
    public void init(BrillienCommunication comm) {
        this.comm = comm;
    }

    protected abstract void sendCommunication( BrillienCommunication comm );

    protected String getSubject(Communication message) {
        return message.getMessageType() == Communication.TYPE_RESULT
                ? message.getRedirectMessage()
                : message.getSubject();
    }

    protected Object communicationReceived() throws BrillienException {
        try{
            String msg = getSubject(comm);
            Method m = ReflectionServices.getMethod(this.getClass(), msg);

            Object[] parameters = null;
            try {
                parameters = comm.acquireParameters(m.getParameterTypes());
            } catch (Throwable t) {
                throw new BrillienException("Cannot map parameters", t);
            }

            return invokeMethod(m, this, parameters);
        } catch( BrillienException be ){
            throw be;
        } catch( Exception e ){
            throw new BrillienException( e );
        }
    }

    protected void processError() {
        try {
            logger.error("Error response received: " + comm.<String>acquireResponse());
        } catch (BrillienException e) {
            logger.error("Error handling error message: " + comm);
        }
    }

    protected void processCall() throws BrillienException {
        if (comm.getMessageType() == Communication.TYPE_GET ||
                comm.getMessageType() == Communication.TYPE_AGET ||
                comm.getMessageType() == Communication.TYPE_DELEGATED_GET ||
                comm.getMessageType() == Communication.TYPE_SET) {
            logger.debug("Processing Get/AGet/DGet/Set message : " + comm.getSubject());

            Object result = communicationReceived();
            if( result != null ){
                comm.prepareForResponse( result );

                sendCommunication( comm );
            }
        } else if (comm.getMessageType() == Communication.TYPE_RESULT) {
            logger.debug("Processing result-type message.");

            if (comm.getOriginalType() == Communication.TYPE_GET) {
                // already processed
            } else if (comm.getOriginalType() == Communication.TYPE_AGET) {
                communicationReceived();
            } else if (comm.getOriginalType() == Communication.TYPE_DELEGATED_GET) {
                communicationReceived();
            }
        }
    }

    @Override
    public void run() {
        try {
            if (comm.getMessageType() == Communication.TYPE_ERROR)
                processError();
            else
                processCall();
        } catch (Throwable t) {
            logger.error("Error processing message: " + comm, t);
        }
    }

}
