/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.sso.Session;
import com.vii.brillien.kernel.axiom.transport.Communication;
import com.vii.streamline.services.TypeServices;
import com.vii.streamline.services.error.StreamLineException;
import com.vii.streamline.services.json.JsonServices;
import com.vii.streamline.structures.collections.InnerList;

import java.util.List;
import java.util.Map;

/**
 * Abstract implementation of Communication defined by module clarity.
 */
public abstract class BrillienCommunication implements Communication<BrillienMessage> {

    /**
     * Message wrapped in this Communicaiton instance
     */
    protected BrillienMessage           message;

    /**
     * Type of the wrapped message.
     */
    protected Class<BrillienMessage>    messageType;

    protected BrillienCommunication( Class<BrillienMessage> messageType ) throws BrillienException {
        this.messageType = messageType;
        try {
            this.message = messageType.newInstance();
        } catch (Exception e) {
            throw new BrillienException( e );
        }
    }


    /**
     * Get method for field message
     */
    public BrillienMessage getMessage() {
        return message;
    }

    /**
     * Set method for field message
     */
    public void setMessage(BrillienMessage message) {
        this.message = message;
    }

    @Override
    public String getId() {
        return message.getId();
    }

    @Override
    public void setId(String id) {
        message.setId( id );
    }

    @Override
    public String getThreadId() {
        return message.getThreadId();
    }

    @Override
    public void setThreadId(String threadId) {
        message.setThreadId( threadId );
    }

    @Override
    public int getMessageType() {
        return message.getMessageType();
    }

    @Override
    public void setMessageType(int messageType) {
        message.setMessageType( messageType );
    }

    @Override
    public String getSender() {
        return message.getSender();
    }

    @Override
    public void setSender(String sender) {
        message.setSender( sender );
    }

    @Override
    public String getRecipient() {
        return message.getRecipient();
    }

    @Override
    public void setRecipient(String recipient) {
        message.setRecipient( recipient );
    }

    @Override
    public String getFlowID() {
        return message.getFlowID();
    }

    @Override
    public void setFlowID(String flowID) {
        message.setFlowID( flowID );
    }

    @Override
    public Session getSession() {
        return message.getSession();
    }

    @Override
    public void setSession(Session session) {
        message.setSession( session );
    }

    @Override
    public String getSubject() {
        return message.getSubject();
    }

    @Override
    public void setSubject(String subject) {
        message.setSubject( subject );
    }

    @Override
    public String getRedirectEntityName() {
        return message.getRedirectEntityName();
    }

    @Override
    public void setRedirectEntityName(String redirectEntityName) {
        message.setRedirectEntityName( redirectEntityName );
    }

    @Override
    public String getRedirectMessage() {
        return message.getRedirectMessage();
    }

    @Override
    public void setRedirectMessage(String redirectMessage) {
        message.setRedirectMessage( redirectMessage );
    }

    @Override
    public Long getRequestTime() {
        return message.getRequestTime();
    }

    @Override
    public void setRequestTime(Long requestTime) {
        message.setRequestTime( requestTime );
    }

    @Override
    public Long getResponseTime() {
        return message.getResponseTime();
    }

    @Override
    public void setResponseTime(Long responseTime) {
        message.setResponseTime( responseTime );
    }

    @Override
    public List<String> getParameters() {
        return message.getParameters();
    }

    @Override
    public void setParameters(List<String> parameters) {
        message.setParameters( parameters );
    }

    @Override
    public String getResponse() {
        return message.getResponse();
    }

    @Override
    public void setResponse(String response) {
        message.setResponse( response );
    }

    @Override
    public int getOriginalType() {
        return message.getOriginalType();
    }

    @Override
    public void setOriginalType(int originalType) {
        message.setOriginalType( originalType );
    }




    @Override
    public Object[] acquireParameters() {
        return  JsonServices.parseJSON(
                message.getParameters()
        ).toArray();
    }

    @Override
    public Object[] acquireParameters(Class[] types) throws BrillienException {
        try{
            String[] params = message.getParameters().toArray( new String[ message.getParameters().size() ] );
            return TypeServices.parse(params, types);
        } catch(Exception e){
            throw new BrillienException( e.getMessage(), e);
        }
    }

    @Override
    public void giveParameters(Object... parameters) throws BrillienException {
        try{
            this.message.setParameters( new InnerList<String>( JsonServices.toJSON(  parameters ) ) );
        } catch(Exception e){
            throw new BrillienException( e.getMessage(), e);
        }
    }

    @Override
    public void giveResponse(Object response) throws BrillienException {
        try{
            this.message.setResponse( JsonServices.toJSON(response) );
            message.setResponseTime( System.currentTimeMillis() );
        } catch(Exception e){
            throw new BrillienException( e.getMessage(), e);
        }
    }

    @Override
    public <T> T acquireResponse() throws BrillienException {
        return JsonServices.<T>parseJSON(message.getResponse());
    }

    @Override
    public <T> T acquireResponse(Class<T> type) throws BrillienException {
        try {
            return (T) TypeServices.parse(message.getResponse(), type);
        } catch (StreamLineException e) {
            throw new BrillienException( e.getMessage(), e );
        }
    }

    @Override
    public <T> T acquireResponse(Class<T> type, Map<String, Class> typeHints) throws BrillienException {
        try {
            return (T) TypeServices.parse(message.getResponse(), type, typeHints);
        } catch (StreamLineException e) {
            throw new BrillienException( e.getMessage(), e );
        }
    }

    @Override
    public <T> T acquireResponse(Class<T> type, String typeHint, Class hintClass) throws BrillienException {
        try {
            return (T)TypeServices.parse( message.getResponse(), type, typeHint, hintClass );
        } catch (StreamLineException e) {
            throw new BrillienException( e.getMessage(), e );
        }
    }

    @Override
    public void prepareForResponse( ) throws BrillienException {
        prepareForResponse( null );
    }

    @Override
    public void prepareForResponse( Object response ) throws BrillienException {
        this.setId( TransportStatics.newMessageId() );

        this.message.setOriginalType( this.message.getMessageType() );
        this.message.setMessageType( TYPE_RESULT );

        String sender = this.message.getRecipient();
        String recipient = this.message.getRedirectEntityName() == null
                    ? message.getSender()
                    : this.message.getRedirectEntityName();
        message.setSender( sender );
        message.setRecipient(recipient);

        if( response != null )
            giveResponse( response );
    }

    @Override
    public void prepareForError(String message) throws BrillienException {
        prepareForResponse();

        this.message.setMessageType( TYPE_ERROR );

        giveResponse(message);
    }

    @Override
    public void processRepresentation(BrillienMessage message) throws BrillienException {
        if( message != null )
            this.setMessage( message );
    }

    @Override
    public BrillienMessage createRepresentation() throws BrillienException {
        return this.message;
    }

    @Override
    public String toString() {
        return "BrillienCommunication{" +
                "message=" + message +
                ", messageType=" + messageType +
                '}';
    }

}
