/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.ignition.BrillienContext;
import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.sso.Session;
import com.vii.brillien.kernel.axiom.transport.Communication;
import com.vii.brillien.kernel.axiom.transport.CommunicationListener;
import com.vii.brillien.kernel.axiom.transport.Mediator;
import com.vii.brillien.kernel.axiom.transport.MessageProcessor;
import com.vii.streamline.services.ThreadServices;
import com.vii.streamline.services.TypeServices;
import com.vii.streamline.services.json.JsonServices;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.logging.Level;

/**
 * Abstract implementation of Mediator defined by the module clarity.
 * It declares the commons functionality for all concrete implementation.
 */
public abstract class BrillienMediator<C extends BrillienCommunication, T> implements Mediator<C, T> {

    /**
     * Queue of registered messageprocessors
     */
    protected BlockingQueue<ProcessorReference>         messageProcessorTypes;

    /**
     * Queue of registered communicationlisteners
     */
    protected BlockingQueue<CommunicationListener<T>>   communicationListeners;

    {
        messageProcessorTypes   = new LinkedBlockingQueue<ProcessorReference>();
        communicationListeners  = new LinkedBlockingQueue<CommunicationListener<T>>();
    }

    /**
     * Creates a new instance of a specific Communication implementation
     */
    protected abstract C newCommunication() throws BrillienException;

    /**
     * Retrieves the String representing the Message to be sent
     */
    protected String getMessageToSend(C comm) throws BrillienException {
        try {
            return JsonServices.toJSON(comm.createRepresentation());
        } catch (Exception e) {
            throw new BrillienException(e);
        }
    }

    @Override
    public <M extends MessageProcessor> void addMessageProcessorType(Class<M> messageProcessorType) {
        messageProcessorTypes.offer( new ProcessorReference<M>( messageProcessorType ) );
    }

    @Override
    public <M extends MessageProcessor> void addMessageProcessorType(Class<M> messageProcessorType, Object outerInstance) {
        messageProcessorTypes.offer( new ProcessorReference<M>( messageProcessorType, outerInstance) );
    }

    @Override
    public <M extends MessageProcessor> Iterable<M> getNewMessageProcessorInstances(C comm) throws BrillienException {
        try{
            LinkedList<M> list = new LinkedList<M>();
            for( ProcessorReference<M> reference : messageProcessorTypes ){
                M processor = reference.newInstance();
                processor.init( comm );
                list.add( processor );
            }
            return list;
        } catch(Exception e){
            throw new BrillienException( e );
        }
    }

    @Override
    public <M extends CommunicationListener<T>> void addCommunicationListener(M messageProcessor) {
        communicationListeners.offer(messageProcessor);
    }

    @Override
    public <M extends CommunicationListener<T>> void removeCommunicationListener(M messageProcessor) {
        communicationListeners.remove(messageProcessor);
    }

    @Override
    public Iterable<CommunicationListener<T>> getCommunicationListeners() {
        return communicationListeners;
    }

    @Override
    public C prepareNewCommunication( ) throws BrillienException {
        C com = newCommunication();

        return com;
    }

    @Override
    public C prepareNewCommunication(String sender, String flowID, String recipient, String subject, int type, Object... parameters) throws BrillienException {
        return prepareNewCommunication((Session) null, sender, flowID, recipient, subject, type, parameters);
    }

    @Override
    public C prepareNewCommunication(C comm, String sender, String flowID, String recipient, String subject, int type, Object... parameters) throws BrillienException {
        return prepareNewCommunication((Session) null, comm, sender, flowID, recipient, subject, type, parameters);
    }

    @Override
    public C prepareNewCommunication(Session session, String sender, String flowID, String recipient, String subject, int type, Object... parameters) throws BrillienException {
        return prepareNewCommunication(session, null, sender, flowID, recipient, subject, type, parameters);
    }

    @Override
    public C prepareNewCommunication(Session session, C comm, String sender, String flowID, String recipient, String subject, int type, Object... parameters) throws BrillienException {
        C newComm = newCommunication();

        if (comm != null) {
            //com.processRepresentation(comm.getMessage());
            newComm.setThreadId(comm.getThreadId());
        }

        newComm.setSession(session);
        newComm.setSender(sender.toLowerCase());
        newComm.setFlowID(flowID);
        newComm.setRecipient(recipient.toLowerCase());
        newComm.setSubject(subject);
        newComm.setMessageType(type);

        if (parameters.length > 0)
            newComm.giveParameters(parameters);

        return newComm;
    }

    @Override
    public void receiveCommunication(C comm) throws BrillienException {
        BrillienContext.basicLog( Level.FINE, mediatorEntity() + ":: " + "internal communication received " + comm );

        for( MessageProcessor processor : getNewMessageProcessorInstances( comm ) ){
            ThreadServices.getStpe().execute( processor );
        }
    }


    @Override
    public C sendCommunication(C comm) throws BrillienException {
        return sendCommunication(BrillienContext.COMMUNICATION_REPLY_TIMEOUT, null, comm);
    }

    @Override
    public C sendCommunication(HashMap<String, Object> properties, C comm) throws BrillienException {
        return sendCommunication(BrillienContext.COMMUNICATION_REPLY_TIMEOUT, properties, comm);
    }

    @Override
    public C sendCommunication(long timeout, C comm) throws BrillienException {
        return sendCommunication(timeout, null, comm);
    }

    protected boolean containsKey( Map<String, Object> parameters, String key ){
        for (String s : parameters.keySet()) {
            if( s.equalsIgnoreCase( key ) )
                return true;
        }
        return false;
    }

    protected Object get( Map<String, Object> parameters, String key ){
        for (String s : parameters.keySet()) {
            if( s.equalsIgnoreCase( key ) )
                return parameters.get( s );
        }
        return null;
    }

    /**
     * Retrieves parameters by mappings
     */
    protected Object[] getParameters( Session session, String to, String message, Map<String, Object> parameters ) throws BrillienException {
        BrillienContext.systemLog(Level.FINE, "Converting map to parameter list to" + to + " about " + message, parameters.toString() );

        C c = sendGet( session, to, "getMessageProcessorParameterNames", message );
        List<String> formalParameterNames = c.acquireResponse( List.class );

        BrillienContext.systemLog(Level.FINE, "Retrieved formalParameterNames", formalParameterNames.toString());

        c = sendGet( session, to, "getMessageProcessorParameterTypes", message );
        List<String> formalParameterTypes = c.acquireResponse( List.class );

        BrillienContext.systemLog(Level.FINE, "Retrieved formalParameterTypes", formalParameterTypes.toString() );

        Object[] os = new Object[ formalParameterNames.size() ];
        for( int i=0; i<os.length; ++i ){
            if( !containsKey( parameters, formalParameterNames.get(i) ) )
                throw new BrillienException( "No parameter found to this formal parameter:" + formalParameterNames.get(i) );

            try{
                Object o = get( parameters, formalParameterNames.get(i) );

                BrillienContext.systemLog(Level.FINE,"Found parameter " + o + " for " + formalParameterNames.get(i));

                os[ i ] =
                        (
                            o instanceof HashMap &&
                            !formalParameterTypes.get(i).equals( HashMap.class.getName() )
                        )
                    ? TypeServices.fillObject((HashMap) o, this.getClass().getClassLoader().loadClass(formalParameterTypes.get(i)))
                    : o;

                BrillienContext.systemLog(Level.FINE, "Added parameter " + os[ i ] );
            } catch( Exception e ){
                throw new BrillienException("Conversion cannot be made: " + formalParameterNames.get(i) + " " + formalParameterTypes.get(i), e );
            }
        }

        return os;
    }

    @Override
    public C sendGet( Session session, String to, String message, Map<String, Object> parameters ) throws BrillienException{
        Object[] os = getParameters( session, to, message, parameters );
        return this.sendGet( session, to, message, os );
    }

    @Override
    public C sendGet( Session session, long timeout, String to, String message, Map<String, Object> parameters ) throws BrillienException{
        Object[] os = getParameters( session, to, message, parameters );
        return this.sendGet( session, timeout, to, message, os );
    }

    @Override
    public void sendAGet( Session session, String to, String message, String redirectMessage, Map<String, Object> parameters ) throws BrillienException{
        Object[] os = getParameters( session, to, message, parameters );
        sendAGet( session, to, message, redirectMessage, os);
    }

    @Override
    public void sendDelegatedGet( Session session, String to, String message, String redirectEntityName, String redirectMessage, Map<String, Object> parameters ) throws BrillienException{
        Object[] os = getParameters( session, to, message, parameters );
        sendDelegatedGet( session, to, message, redirectEntityName, redirectMessage, os);
    }

    @Override
    public void sendSet( Session session, String to, String message, Map<String, Object> parameters ) throws BrillienException{
        Object[] os = getParameters( session, to, message, parameters );
        sendSet( session, to, message, os);
    }

    @Override
    public C sendGet( Session session, String to, String message, Object... parameters ) throws BrillienException{
        return sendGet( session, BrillienContext.COMMUNICATION_REPLY_TIMEOUT, to, message, parameters);
    }

    @Override
    public C sendGet( Session session, long timeout, String to, String message, Object... parameters ) throws BrillienException{
        C rpc = prepareNewCommunication( session,
                mediatorEntity( ), null,
                to, message, Communication.TYPE_GET, parameters
        );

        return sendCommunication(timeout, rpc);
    }


    @Override
    public void sendAGet( Session session, String to, String message, String redirectMessage, Object... parameters ) throws BrillienException{
        C rpc = prepareNewCommunication( session,
                mediatorEntity( ), null,
                to, message, Communication.TYPE_AGET, parameters
        );
        rpc.setRedirectMessage(redirectMessage);

        sendCommunication(rpc);
    }


    @Override
    public void sendDelegatedGet( Session session, String to, String message, String redirectEntityName, String redirectMessage, Object... parameters ) throws BrillienException{
        C rpc = prepareNewCommunication( session,
                mediatorEntity( ), null,
                to, message, Communication.TYPE_DELEGATED_GET, parameters
        );
        rpc.setRedirectEntityName( redirectEntityName );
        rpc.setRedirectMessage( redirectMessage );

        sendCommunication(rpc);
    }

    @Override
    public void sendSet( Session session, String to, String message, Object... parameters ) throws BrillienException{
        C rpc = prepareNewCommunication( session,
                mediatorEntity( ), null,
                to, message, Communication.TYPE_SET, parameters
        );

        sendCommunication(rpc);
    }


}
