/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.kernel.axiom.sso.Session;
import com.vii.brillien.kernel.axiom.transport.Communication;
import com.vii.brillien.kernel.axiom.transport.Message;
import org.svenson.JSONProperty;

import java.util.LinkedList;
import java.util.List;

/**
 * Simple implementation for the Message axiom
 */
public class BrillienMessage implements Message {

    /**
     * Unique id of the message
     */
    private String                  id;

    /**
     * Unique id of the message
     */
    private String                  threadId;

    /**
     * Unique name of the sender
     */
    private String                  sender;

    /**
     * Unique name of the recipient
     */
    private String                  recipient;

    /**
     * Stores the type of the original message if this is a response.
     */
    private int                     messageType;

    /**
     * ID of the given flow instance
     */
    private String                  flowID;

    /**
     * ID of the given session where this message participate
     */
    private Session                 session;

    /**
     * Name of the message
     */
    private String                  subject;


    /**
     * Name of the redirection recipient entity if exists.
     */
    private String                  redirectEntityName;

    /**
     * Name of the return message if exists.
     */
    private String                  redirectMessage;

    /**
     * Timestamp marking the date of sending.
     */
    private Long                    requestTime;

    /**
     * Timestamp marking the date of responding.
     */
    private Long                    responseTime;

    /**
     * Parameters of the message.
     */
    private List<String>            parameters;

    /**
     * Response object if exits.
     */
    private String                  response;

    /**
     * Stores the type of the original packet if this is a response.
     */
    private int                     originalType;


    {
        id = TransportStatics.newMessageId();

        threadId = id;

        messageType = originalType = Communication.TYPE_UNDEFINED;
        parameters = new LinkedList<String>();

        setRequestTime( System.currentTimeMillis() );
        setResponseTime( new Long(-1) );

        flowID = DEFAULT_FLOW_ID;
    }

    /**
     * Getter method for field id
     */
    @JSONProperty(ignoreIfNull = true)
    public String getId() {
        return id;
    }

    /**
     * Setter method for field id
     */
    public void setId(String id) {
        this.id = id;
    }


    public String getThreadId() {
        return threadId;
    }

    public void setThreadId(String threadId) {
        this.threadId = threadId;
    }

    /**
     * Getter method for field originalType
     */
    public int getMessageType() {
        return messageType;
    }

    /**
     * Setter method for field originalType
     */
    public void setMessageType(int messageType) {
        this.messageType = messageType;
    }

    /**
     * Getter method for field sender
     */
    @JSONProperty(ignoreIfNull = true)
    public String getSender() {
        return sender;
    }

    /**
     * Setter method for field sender
     */
    public void setSender(String sender) {
        this.sender = sender;
    }

    /**
     * Getter method for field recipientName
     */
    @JSONProperty(ignoreIfNull = true)
    public String getRecipient() {
        return recipient;
    }

    /**
     * Setter method for field recipient
     */
    public void setRecipient(String recipient) {
        this.recipient = recipient;
    }

    /**
     * Getter method for field flowID
     */
    @JSONProperty(ignoreIfNull = true)
    public String getFlowID() {
        return flowID;
    }

    /**
     * Setter method for field flowID
     */
    public void setFlowID(String flowID) {
        this.flowID = (flowID == null || flowID.equals(""))
                ? DEFAULT_FLOW_ID
                : flowID;
    }

    /**
     * Getter method for field session
     */
    @JSONProperty(ignoreIfNull = true)
    public Session getSession() {
        return session;
    }

    /**
     * Setter method for field session
     */
    public void setSession(Session session) {
        this.session = session;
    }

    /**
     * Getter method for field subject
     */
    public String getSubject() {
        return subject;
    }

    /**
     * Setter method for field subject
     */
    public void setSubject(String subject) {
        this.subject = subject;
    }

    /**
     * Getter method for field redirectEntityName
     */
    @JSONProperty(ignoreIfNull = true)
    public String getRedirectEntityName() {
        return redirectEntityName;
    }

    /**
     * Setter method for field redirectEntityName
     */
    public void setRedirectEntityName(String redirectEntityName) {
        this.redirectEntityName = redirectEntityName;
    }

    /**
     * Getter method for field redirectMessage
     */
    @JSONProperty(ignoreIfNull = true)
    public String getRedirectMessage() {
        return redirectMessage;
    }

    /**
     * Setter method for field redirectMessage
     */
    public void setRedirectMessage(String redirectMessage) {
        this.redirectMessage = redirectMessage;
    }

    /**
     * Getter method for field requestTime
     */
    public Long getRequestTime() {
        return requestTime;
    }

    /**
     * Setter method for field requestTime
     */
    public void setRequestTime(Long requestTime) {
        this.requestTime = requestTime;
    }

    /**
     * Getter method for field responseTime
     */
    @JSONProperty(ignoreIfNull = true)
    public Long getResponseTime() {
        return responseTime;
    }

    /**
     * Getter method for field responseTime
     */
    public void setResponseTime(Long responseTime) {
        this.responseTime = responseTime;
    }

    /**
     * Getter method for field parameters
     */
    @JSONProperty(ignoreIfNull = true)
    public List<String> getParameters() {
        return parameters;
    }

    /**
     * Setter method for field parameters
     */
    public void setParameters(List<String> parameters) {
        this.parameters = parameters;
    }

    /**
     * Getter method for field response
     */
    @JSONProperty(ignoreIfNull = true)
    public String getResponse() {
        return response;
    }

    /**
     * Setter method for field response
     */
    public void setResponse(String response) {
        this.response = response;
    }

    /**
     * Getter method for field originalType
     */
    @JSONProperty(ignoreIfNull = true)
    public int getOriginalType() {
        return originalType;
    }

    /**
     * Setter method for field originalType
     */
    public void setOriginalType(int originalType) {
        this.originalType = originalType;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (String parameter : parameters) {
            sb.append( parameter + " " );
        }
        return "BrillienMessage{" +
                "id='" + id + '\'' +
                ", sender='" + sender + '\'' +
                ", subject='" + subject + '\'' +
                ", recipient='" + recipient + '\'' +
                ", threadId='" + threadId + '\'' +
                ", messageType=" + messageType +
                ", flowID='" + flowID + '\'' +
                ", session=" + session +
                ", redirectEntityName='" + redirectEntityName + '\'' +
                ", redirectMessage='" + redirectMessage + '\'' +
                ", requestTime=" + requestTime +
                ", responseTime=" + responseTime +
                ", parameters=" + sb +
                ", response='" + response + '\'' +
                ", originalType=" + originalType +
                '}';
    }
}
