/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport.mq;

import com.vii.brillien.kernel.BrillienException;
import com.vii.streamline.services.ThreadServices;

import java.util.HashMap;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * Utility class implementing a Queue-based message management used by MqMediator instances.
 */
public class MqServices {

    /**
     * Queue where the new messages will be posted.
     */
    private static BlockingQueue<MqCommunication>       queue;

    /**
     * Processor instance handling all incoming messages.
     */
    private static QueueProcessor                       processor;

    /**
     * Map managing the queues and its mediator instances associated with.
     */
    private static HashMap<String, MqMediator>          queues;

    static{
        queue       = new LinkedBlockingQueue<MqCommunication>();
        processor   = new QueueProcessor();
        queues      = new HashMap<String, MqMediator>();
    }

    private static class QueueProcessor implements Runnable{
        private boolean working;
        @Override
        public void run() {
            working = true;

            while( !queue.isEmpty() ){
                try {
                    MqCommunication comm = queue.take();
                    if( queues.containsKey( comm.getRecipient() ) ){
                        try {
                            queues.get( comm.getRecipient() ).receiveCommunication( comm );
                        } catch (BrillienException e) {
                            e.printStackTrace();
                        }
                    }
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            working = false;
        }
        public void newMessageArrived(){
            if(!working)
                ThreadServices.getStpe().submit( this );
        }
    }

    /**
     * Registers a Mediator to the given queue
     */
    public static void registerMediator( String queue, MqMediator mediator ){
        queues.put( queue, mediator );
    }

    /**
     * Unregisters the mediator associated to the given queue
     */
    public static void unregisterMediator( String queue ){
        queues.remove( queue );
    }

    /**
     * Publishes a new communication invoked by the registered MqMediator instances.
     */
    public static void publish(MqCommunication comm) throws BrillienException{
        boolean res = queue.offer( comm );
        if( !res )
            throw new BrillienException( "Queue is full, cannot deliver new messages!" );

        processor.newMessageArrived();
    }

}
